from bunch import Bunch
from copy import deepcopy
from blockerbugs.util.update_sync import UpdateSync
from mock import MagicMock
from datetime import datetime


basicupdate = Bunch(
    approved=None,
    close_bugs=True,
    critpath=False,
    critpath_approved=True,
    date_modified=None,
    date_pushed=u'2012-09-13 16:40:28',
    date_submitted=u'2012-09-12 23:43:04',
    notes=u'Fix live install mounting /home and /boot and swap',
    request=None, stable_karma=3, status=u'testing',
    submitter=u'bcl',
    title=u'anaconda-18.6.8-1.fc18',
    type=u'bugfix',
    karma=1,
    unstable_karma=-3,
    updateid=u'FEDORA-2012-13902',
    bugs=[Bunch(bz_id=856836, parent=False, security=False,
                title=u'/etc/fstab is not written correctly after live install (F18 Alpha RC2+)')],
    builds=[Bunch(nvr=u'anaconda-18.6.8-1.fc18',
                  package=Bunch(
                      committers=[u'anaconda-maint', u'dlehman', u'jkeating',
                                  u'bcl', u'clumens', u'rvykydal', u'akozumpl',
                                  u'pjones',
                                  u'katzj',
                                  u'dcantrel'],
                      name=u'anaconda',
                      suggest_reboot=False))],
    comments=[
        Bunch(anonymous=False, author=u'bodhi', group=None,
              karma=0,
              text=u'This update has been submitted for testing by bcl. ',
              timestamp=u'2012-09-12 23:43:24',
              update_title=u'anaconda-18.6.8-1.fc18'),
        Bunch(anonymous=False, author=u'autoqa', group=None,
              karma=0,
              text=u'AutoQA: depcheck test PASSED on x86_64. Result log:\nhttp://autoqa.fedoraproject.org/report/em84 (results are informative only)',
              timestamp=u'2012-09-13 00:14:41',
              update_title=u'anaconda-18.6.8-1.fc18'),
        Bunch(anonymous=False, author=u'autoqa', group=None,
              karma=0,
              text=u'AutoQA: depcheck test PASSED on i386. Result log:\nhttp://autoqa.fedoraproject.org/report/emcl (results are informative only)',
              timestamp=u'2012-09-13 00:30:53',
              update_title=u'anaconda-18.6.8-1.fc18'),
        Bunch(anonymous=False, author=u'jkeating',
              group=u'proventesters', karma=1,
              text=u'While I did not test the above bug, I have tested anaconda in a number of other\nscenarios and it performs as expected.',
              timestamp=u'2012-09-13 06:27:03',
              update_title=u'anaconda-18.6.8-1.fc18'),
        Bunch(anonymous=False, author=u'bodhi', group=None,
              karma=0,
              text=u'This update is currently being pushed to the Fedora 18 testing updates repository.',
              timestamp=u'2012-09-13 15:58:32',
              update_title=u'anaconda-18.6.8-1.fc18'),
        Bunch(anonymous=False, author=u'bodhi', group=None,
              karma=0,
              text=u'This update has been pushed to testing',
              timestamp=u'2012-09-13 16:44:30',
              update_title=u'anaconda-18.6.8-1.fc18')],
    nagged=Bunch(
        test_cases=[
            u'QA:Testcase Anaconda autopart install',
            u'QA:Testcase Anaconda User Interface Graphical',
            u'QA:Testcase Boot Methods Pxeboot',
            u'QA:Testcase install repository Live Image',
        ]),
    release=Bunch(dist_tag=u'f18', id_prefix=u'FEDORA',
                  locked=True,
                  long_name=u'Fedora 18', name=u'F18')
)


class TestUpdateSyncExtractInformation(object):
    def setup_method(self, method):
        self.testupdate = deepcopy(basicupdate)
        self.testsync = UpdateSync(None, MagicMock())

    def test_extract_status_testing(self):
        self.testupdate.status = u'testing'

        updateinfo = self.testsync.extract_information(self.testupdate)

        assert updateinfo['status'] == 'testing'
        assert updateinfo['pending'] == False

    def test_extract_status_pending_stable(self):
        self.testupdate.status = u'pending'
        self.testupdate.request = u'stable'

        updateinfo = self.testsync.extract_information(self.testupdate)

        assert updateinfo['status'] == 'stable'
        assert updateinfo['pending'] == True

    def test_extract_status_pending_testing(self):
        self.testupdate.status = u'pending'
        self.testupdate.request = u'testing'

        updateinfo = self.testsync.extract_information(self.testupdate)

        assert updateinfo['status'] == 'testing'
        assert updateinfo['pending'] == True

    def test_extract_status_pending_nothing(self):
        self.testupdate.status = u'pending'
        self.testupdate.request = None

        updateinfo = self.testsync.extract_information(self.testupdate)

        assert updateinfo['status'] == 'undefined'
        assert updateinfo['pending'] == True

    def test_extract_title(self):
        ref_title = u'magic-fixall-update-1.1-0'
        self.testupdate.title = ref_title

        updateinfo = self.testsync.extract_information(self.testupdate)

        assert updateinfo['title'] == ref_title

    def test_extract_karma(self):
        ref_karma = 2
        self.testupdate.karma = ref_karma

        updateinfo = self.testsync.extract_information(self.testupdate)

        assert updateinfo['karma'] == ref_karma

    def test_extract_url(self):
        updateinfo = self.testsync.extract_information(self.testupdate)

        assert (updateinfo['url'] == 'https://admin.stg.fedoraproject.org/updates/%s' % self.testupdate.updateid)

    def test_extract_url_no_updateid(self):
        ref_title = u'magic-fixall-update-1.1-0'
        self.testupdate.title = ref_title
        self.testupdate.updateid = None

        updateinfo = self.testsync.extract_information(self.testupdate)

        baseurl = 'https://admin.stg.fedoraproject.org/updates/%s'

        assert (updateinfo['url'] == 'https://admin.stg.fedoraproject.org/updates/%s' % ref_title)

    def test_extract_date_pushed_testing_intesting(self):
        self.testupdate.status = u'testing'
        self.testupdate.request = None
        self.testupdate.date_pushed = u'2012-09-13 16:40:28'

        updateinfo = self.testsync.extract_information(self.testupdate)

        assert updateinfo['date_pushed_testing'] == datetime.strptime(self.testupdate.date_pushed, '%Y-%m-%d %H:%M:%S')
        assert updateinfo['date_pushed_stable'] == None

    def test_extract_date_pushed_testing_pendingstable(self):
        self.testupdate.status = u'pending'
        self.testupdate.request = u'stable'
        self.testupdate.date_pushed = u'2012-09-13 16:40:28'

        updateinfo = self.testsync.extract_information(self.testupdate)

        assert updateinfo['date_pushed_testing'] == datetime.strptime(self.testupdate.date_pushed, '%Y-%m-%d %H:%M:%S')
        assert updateinfo['date_pushed_stable'] == None

    def test_extract_date_pushed_stable(self):
        self.testupdate.status = u'stable'
        self.testupdate.request = None
        self.testupdate.date_pushed = u'2012-09-13 16:40:28'

        updateinfo = self.testsync.extract_information(self.testupdate)

        assert updateinfo['date_pushed_testing'] == None
        assert updateinfo['date_pushed_stable'] == datetime.strptime(self.testupdate.date_pushed, '%Y-%m-%d %H:%M:%S')

    def test_extract_date_pushed_pending_testing(self):
        self.testupdate.status = u'pending'
        self.testupdate.request = u'testing'
        self.testupdate.date_pushed = None

        updateinfo = self.testsync.extract_information(self.testupdate)

        assert updateinfo['date_pushed_testing'] == None
        assert updateinfo['date_pushed_stable'] == None

    def test_extract_bug_single(self):
        ref_bugs = [Bunch(bz_id=123456, parent=False, security=False,
            title=u'First wonderful test bug')
         ]
        self.testupdate.bugs = ref_bugs

        updateinfo = self.testsync.extract_information(self.testupdate)

        assert len(updateinfo['bugs']) == 1
        assert updateinfo['bugs'][0] == 123456

    def test_extract_bug_multiple(self):
        ref_bugs = [
            Bunch(bz_id=123456, parent=False, security=False,
                  title=u'First wonderful test bug'),
            Bunch(bz_id=234567, parent=False, security=False,
                  title=u'Second wonderful test bug'),
            Bunch(bz_id=345678, parent=False, security=False,
                  title=u'Third wonderful test bug'),
        ]
        self.testupdate.bugs = ref_bugs

        updateinfo = self.testsync.extract_information(self.testupdate)

        assert len(updateinfo['bugs']) == 3
        assert 123456 in updateinfo['bugs']
        assert 234567 in updateinfo['bugs']
        assert 345678 in updateinfo['bugs']

    def test_extract_date_submitted(self):
        ref_date_submitted = u'2012-09-12 23:43:04'
        self.testupdate.date_submitted = ref_date_submitted

        updateinfo = self.testsync.extract_information(self.testupdate)

        assert updateinfo['date_submitted'] == datetime.strptime(ref_date_submitted, '%Y-%m-%d %H:%M:%S')

    # check for handling of update data when there is no date_pushed info which
    # occurs when a brand new package is submitted as a blocker or FE fix
    def test_extract_pushed_date_newpackage(self):
        self.testupdate.date_pushed = None

        self.testsync.extract_information(self.testupdate)
